"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const sprintf_js_1 = require("sprintf-js");
const LogLevel_1 = require("../../model/logging/LogLevel");
const Environment_1 = require("../Environment");
/**
 * Logger implementation that uses the console for output.
 */
class ConsoleLogger {
    /**
     * Build a new logger.
     *
     * @param clazz The enclosing class that will use the new logger.
     * @param logLevel Lowest level to log, defaults to `info`.
     * @param logFormat Format to output log messages in, defaults to `string`.
     * @param logTimestamp Print an ISO 8601 timestamp before every log message? (string format only)
     */
    constructor(clazz, level, format, logTimestamp) {
        this.clazz = clazz;
        this.level = level;
        this.format = format;
        this.logTimestamp = logTimestamp;
        this.useStringLogFormat = this.format === "string";
    }
    formatMessage(message, ...formatArgs) {
        try {
            return sprintf_js_1.sprintf(message, ...formatArgs);
        }
        catch (ex) {
            this.errorWithStack("Failed to format log message, raw log message and parameters will be output", ex);
            // fall back to raw log message and args dump using inspect
            return `${message}\n${util_1.inspect(formatArgs)}`;
        }
    }
    /**
     * Log a message with a custom level. Ignores messages of a lower log
     * level than the current level. If [[Environment.UnderTest]] is enabled
     * then this method will only log info messages or higher.
     *
     * @param level Level of the log message.
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    log(level, message, ...formatArgs) {
        if (level < this.level) {
            return;
        }
        let uppercaseLevel = LogLevel_1.LogLevel[level].toUpperCase();
        let now = new Date();
        let formattedMessage = message;
        if (formatArgs && formatArgs.length > 0) {
            formattedMessage = this.formatMessage(message, ...formatArgs);
        }
        let logLine = this.useStringLogFormat ?
            sprintf_js_1.sprintf("%s%s %s - %s", this.logTimestamp ? `${now.toISOString()} ` : "", uppercaseLevel, this.clazz, formattedMessage) :
            JSON.stringify({
                level: uppercaseLevel,
                msg: formattedMessage,
                time: now.getTime()
            });
        if (Environment_1.UnderTest && level < LogLevel_1.LogLevel.info) {
            // prevent verbose logging when running under test,
            // build the message to test that, but only output it
            // if it is a warning or an error
            return;
        }
        console.log(logLine);
    }
    /**
     * Log a trace level message.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    trace(message, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.trace, message, ...formatArgs);
    }
    /**
     * Log a debug level message.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    debug(message, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.debug, message, ...formatArgs);
    }
    /**
     * Log a info level message.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    info(message, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.info, message, ...formatArgs);
    }
    /**
     * Log a warn trace level message.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    warn(message, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.warn, message, ...formatArgs);
    }
    /**
     * Log a error level message.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    error(message, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.error, message, ...formatArgs);
    }
    /**
     * Log a error level message with an associated error and stack trace.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param ex Error object associated with this error message.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    errorWithStack(message, ex, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.error, `${message}\n${ex ? ex.stack : ""}`, ...formatArgs);
    }
    /**
     * Log a fatal level message.
     *
     * @param message String which can contain `sprintf` style format placeholders.
     * @param formatArgs (Optional) Arguments which are passed to `sprintf` to format the message.
     */
    fatal(message, ...formatArgs) {
        this.log(LogLevel_1.LogLevel.fatal, message, ...formatArgs);
    }
    /**
     * Is a given log level enabled?
     *
     * @param level Level to check.
     */
    levelEnabled(level) {
        if (level === LogLevel_1.LogLevel.off) {
            return this.level === level;
        }
        return this.level <= level;
    }
    /**
     * Is trace log level enabled?
     */
    traceEnabled() {
        return this.levelEnabled(LogLevel_1.LogLevel.trace);
    }
    /**
     * Is debug log level enabled?
     */
    debugEnabled() {
        return this.levelEnabled(LogLevel_1.LogLevel.debug);
    }
    /**
     * Is info log level enabled?
     */
    infoEnabled() {
        return this.levelEnabled(LogLevel_1.LogLevel.info);
    }
    /**
     * Is warn log level enabled?
     */
    warnEnabled() {
        return this.levelEnabled(LogLevel_1.LogLevel.warn);
    }
    /**
     * Is error log level enabled?
     */
    errorEnabled() {
        return this.levelEnabled(LogLevel_1.LogLevel.error);
    }
    /**
     * Is fatal log level enabled?
     */
    fatalEnabled() {
        return this.levelEnabled(LogLevel_1.LogLevel.fatal);
    }
    /**
     * Is the current log level `off`?
     */
    isOff() {
        return this.levelEnabled(LogLevel_1.LogLevel.off);
    }
}
exports.ConsoleLogger = ConsoleLogger;
//# sourceMappingURL=ConsoleLogger.js.map