"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AppConfig_1 = require("../../model/AppConfig");
const LogLevel_1 = require("../../model/logging/LogLevel");
const ConsoleLogger_1 = require("./ConsoleLogger");
/**
 * Builds implementations of [[ILogger]].
 */
class LogFactory {
    /**
     * Build a new log factory using application config.
     *
     * @param appConfig Config object to read logging config from.
     * @param logLevel (Optional) Lowest level to log, defaults to `info`.
     * @param logFormat (Optional) Format to output log messages in, defaults to `string`.
     * @param logTimestamp (Optional) Print an ISO 8601 timestamp before every log message?
     *                     (string format only, defaults to `false`).
     */
    constructor(appConfig, logLevel = LogLevel_1.LogLevel.info, logFormat = "string", logTimestamp = false) {
        this.logLevel = logLevel;
        this.logFormat = logFormat;
        this.logTimestamp = logTimestamp;
        if (appConfig && appConfig.serverLogger) {
            if (appConfig.serverLogger.level) {
                this.logLevel = appConfig.serverLogger.level;
            }
            if (appConfig.serverLogger.format) {
                this.logFormat = appConfig.serverLogger.format;
            }
            if (appConfig.serverLogger.logTimestamp !== null &&
                appConfig.serverLogger.logTimestamp !== undefined) {
                this.logTimestamp = appConfig.serverLogger.logTimestamp;
            }
        }
    }
    /**
     * Create a new logger.
     *
     * @param clazz The enclosing class that will use the new logger.
     */
    getLogger(clazz) {
        return new ConsoleLogger_1.ConsoleLogger(clazz ? clazz.name : "?", this.logLevel, this.logFormat, this.logTimestamp);
    }
    /**
     * Create a new logger using [[AppConfig]] config defaults.
     *
     * @param clazz The enclosing class that will use the new logger.
     */
    static getDefaultLogger(clazz) {
        let logFactory = new LogFactory(new AppConfig_1.AppConfig());
        return logFactory.getLogger(clazz);
    }
    /**
     * Create a new logger using custom log configuration.
     *
     * @param clazz The enclosing class that will use the new logger.
     * @param logLevel (Optional) Lowest level to log, defaults to `info`.
     * @param logFormat (Optional) Format to output log messages in, defaults to `string`.
     * @param logTimestamp (Optional) Print an ISO 8601 timestamp before every log message?
     *                     (string format only, defaults to `false`).
     */
    static getCustomLogger(clazz, level = LogLevel_1.LogLevel.info, format = "string", logTimestamp = false) {
        let logFactory = new LogFactory({
            serverLogger: {
                format,
                level,
                logTimestamp
            }
        });
        return logFactory.getLogger(clazz);
    }
}
exports.LogFactory = LogFactory;
//# sourceMappingURL=LogFactory.js.map